#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h> 
#include <fridge.h>
#include <sys/wait.h>


#define MAX_KEY_SIZE 200

void blocking_multiple_test(int num_proc){

  /*
   * multiple processes blocked on the same (non-existent) key
   * parent inserts a value for the key
   * one process wakes up, the others still wait
   */
  int key = 0xdead;
  char *orange = "orange";
  char res[MAX_KEY_SIZE];
  int ret = 0;
  pid_t pid;
  int count = num_proc;
  pid_t *pids = NULL;
  int idx = 0;
  
  pids = (pid_t*)malloc(count*sizeof(pid_t));
  if (!pids) {
    printf("error:%s\n", strerror(errno));
    exit(0);
  }
  
  memset(res, 0x0, sizeof res);
  while(count--) {
    pid = fork();
    if (pid < 0) {
      printf("error:%s\n",strerror(errno));
      return;
    } else if (pid == 0) {
      // child - make a blocking call
      if(pids)
	free(pids);
      pid = getpid();
      ret = kkv_get((int)pid, res, MAX_KEY_SIZE, 1);
      assert(ret == 0);
      assert(strcmp(orange,res) == 0);
      exit(0);
    } else {
      pids[idx++] = pid;
    }
  }
  sleep(1);
  count = num_proc;
  for (idx=0; idx<count; ++idx) {
    // unblock children one by one
    ret = kkv_put((int)pids[idx], orange, strlen(orange)+1, 1);
    if (ret) {
      printf("kkv_put failed!\n");
      assert(1 == 0);
    }
  }
  free(pids);
  // reap all children
  while(pid = waitpid(-1, NULL, 0)) {
    if (errno == ECHILD)
      break;
  }
}
