#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/syscall.h>
#include <unistd.h> 
#include <fridge.h>
#include <signal.h>
#include <sys/wait.h>

#define MAX_KEY_SIZE 200

static int key = 0xdead;
static char *orange = "orange";
static char *apple = "apple";
static char *banana = "banana";
static char res[MAX_KEY_SIZE];
static int sig_test = 0;

/*
 * signal handler function
 * for SIGINT
 */
static void signal_handler(int signum) {
  sig_test  =1;
}

void test1()
{
  int ret = 0;
  pid_t pid;
  
  memset(res, 0x0, sizeof res);
    /*
   * fork a process
   * child blocks on a non-existent value
   * parent inserts the value into the hash table
   * child wakes up from sleep
   */
  pid = fork();
  if (pid < 0) {
    printf("error:%s\n", strerror(errno));
    return ;
  } else if (pid == 0) {
    // child process - block
    ret = kkv_get(key, res, MAX_KEY_SIZE, 1);
    assert(strcmp(orange,res) == 0);
    exit(0); 
  } else {
    // parent- intentionally sleep to allow child to block
    usleep(100);
    ret = kkv_put(key, orange, strlen(orange)+1, 1);
    if (ret) {
      printf("error in kkv_put\n");    
    }
    // reap child process
    ret = wait(NULL);
    if (ret == -1) {
      printf("error: %s\n", strerror(errno));
      return;
    }
  }

}


void test2(){
/*
   * simple signal test
   * block the child on kkv_get
   * send a signal to the child, it should wake up
   */
   int ret = 0;
   pid_t pid;
  
   memset(res, 0x0, sizeof res);

   sig_test = 0;
   pid = fork();
   if (pid < 0) {
     printf("error:%s\n", strerror(errno));
     return ;
   } else if (pid == 0) {
     // child
     ret = kkv_get(key, res, MAX_KEY_SIZE, 1);
     assert(ret < 0);
     assert(sig_test == 1);
     exit(0);
   } else {
     // parent
     usleep(100);
     ret = kill(pid, SIGINT);
     if (ret) {
       printf("error: %s\n", strerror(errno));
       return ;
     }
     // reap child
     ret = wait(NULL);
     if (ret == -1) {
       printf("error:%s\n", strerror(errno));
     }
   }   
}


void test3() {
    /*
     * block child process on a key
     * insert a different key via the parent
     * child should not wakeup
     * to assert this, the user provides a SIGINT, and the handler sets the sig_test to 1
     */
  
  int ret = 0;
  pid_t pid;
  
  memset(res, 0x0, sizeof res);
     
  sig_test = 0;
  pid = fork();
  if (pid < 0) {
    printf("error:%s\n", strerror(errno));
    return ;
  } else if (pid == 0) {
    // child process - block
    
    ret = kkv_get(key, res, MAX_KEY_SIZE, 1);
    // ideally if we came here without a signal, then there is an error
    assert(sig_test == 1);
    exit(0);
  } else {
    // inserting a different key here
    ret = kkv_put(key+1, orange, strlen(orange)+1, 1);
    if (ret) {
      printf("kkv_put failed\n");
      assert(1 == 0);
    }
    usleep(100);
    // send a signal to the child
    
    ret = kill(pid, SIGINT);
    if (ret) {
      printf("error:%s\n", strerror(errno));
      assert(1 == 0);
    }
    
    // reap child process
      ret = wait(NULL);
      if (ret == -1) {
	printf("error:%s\n", strerror(errno));
      }
  }
}

void test4() {
  /*
   * test unloading module when process is sleeping
   */
  int ret = 0;
  pid_t pid;
  
  memset(res, 0x0, sizeof res);
  pid = fork();
  if (pid < 0) {
    printf("error:%s\n", strerror(errno));
    return ;
  } else if (pid == 0) {
    // child
     ret = kkv_get(key, res, MAX_KEY_SIZE, 1);
     exit(0);
  } else {
    // parent
    ret = kkv_destroy(0);
    if (ret < 0) {
      printf("kkv_destroy failed!\n");
      assert(1 == 0);
     }
    // reap child
    ret = wait(NULL);
    if(ret == -1)
      printf("error:%s\n", strerror(errno));
   }
}



 /*
   * This function tests the case where we have a process
   * that blocks on a certain non-existent value
   * we have one child process waiting for a non-existent value
   * one parent process inserting a value for a key, that the child
   * is blocked on, thereby releasing the child process
   */
void blocking_sequential_test(void) {
  /*
   * register a signal handler for SIGINT
   */
  int ret;
  if(signal(SIGINT, signal_handler) == SIG_ERR) {
    printf("error:%s\n", strerror(errno));
    return;
  }
  test1();
  test2();
  test3();
  test4();
}




